/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.s2s.generics;

import  java.util.function.*;

/*
 *
 * @author Selvyn
 */
public class GenericMethods 
{
    public  static  void    main( String[] args )
    {
        GenericMethods gm = new GenericMethods();
        
        gm.setup_v4();
        
    }
    
    public  void    setup_v1()
    {
        // version 1 using a variable
        Callback cc = new Callback()
        {
            @Override
            public void execute()
            {
                System.out.println("setup_v1 with a variable");
            }
        };
        
        this.execute_v1(cc);
        
        // version 2 using no variable
        this.execute_v1(new Callback()
        {
            @Override
            public void execute()
            {
                System.out.println("setup_v1 with NO variable");
            }
        });
    }
    
    public  void    setup_v2()
    {
        // The anonymous instance stored in a variable using lambda expression
        Callback cc = ()->System.out.println( "setup_v2 lambda with variable");
        this.execute_v1(cc);
        
        // The anonymous instance not stored in a variable using lambda expression
        this.execute_v1(()->System.out.println("setup_v2 lambda with no variable"));
                
        // The anonymous instance not stored in a variable using multi-line lambda expression
        this.execute_v1(()->{
            System.out.println("[1]setup_v2 lambda with no variable");
            System.out.println("[2]A compound block");
        });
    }
    
    public  void    setup_v3()
    {
        // We know that generics are really functors and can be typed against
        // an interface, so we can create a function that takes as an argument
        // the interface types, see method execute_v1()
        Callback cc = ()->System.out.println( "Hello world" );
        
        execute_v1( cc );
        
        execute_v1( ()->System.out.println( "Hello world") );

        // Java can't resolve this next call, see notes below
        //execute_v2( ()->System.out.println( "Hello world") );

        execute_v3( ()->System.out.println( "Hello world") );
        int a=3, b=7;
        CallbackWithTest ct = (x, y)-> x > y;
        
        execute_v4(a, b, ct);
        
        execute_v4(a, b, ((x,y)-> x > y) );
    }
    
    // working with common functional interfaces
    public  void    setup_v4()
    {
        BiPredicate<Integer, Integer> cc = (x,y)-> x > y;
        this.execute_v5(44, 33, cc);
    }
    
    // This method compiles comfortably because the java compiler knows the 
    // parameter type.
    void    execute_v1( Callback p )
    {
        p.execute();
    }

    // In this example, unlike the C++ language Java cannot infer the type of 
    // paramter p and doesn't support the concept of a two pass compile as it
    // does in C++.  The first pass marks the method as incomplete and that
    // the object being passed in must support the execute method, the second
    // pass verifies that this is the case
    /*
    <T> void    execute_v2( T p )
    {
        p.execute();
    }
    */
    
    // The third approach solves execute_v2's issue.  But using a genereic method
    // we mark the method with a late binding type but we indicate that it must
    // be a classifier that satisfies the requirement of being a subclass of
    // or being of the type functional interface Callback
    <T extends Callback> void    execute_v3( T p )
    {
        p.execute();
    }
        
    void    execute_v4( int x, int y, CallbackWithTest p )
    {
        p.condition(x, y);
    }
    
    void    execute_v6( )
    {
        
    }
    
    void    execute_v5( int x, int y, BiPredicate p)
    {
        if( p.test(x, y) )
            System.out.printf( "%d is greater then %d", x, y);
    }

}

