package s2s.javacourse;



/**
 * Title:        Java Courseware
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      TROy Software Development
 * @author Selvyn Wright
 * @version 1.0
 */

import   java.io.*;
import   java.lang.ClassLoader;
import   java.util.Hashtable;


public class SimpleClassLoader   extends  ClassLoader
{
    private   String root;  // base path
    private   Hashtable   types = new Hashtable();

    public SimpleClassLoader (String rootDir)
    {
        if (rootDir == null)
            throw new IllegalArgumentException ("Null root directory");
        root = rootDir;
    }

    public Class loadClass (String name, boolean resolve)
                                             throws ClassNotFoundException
   {
        // Since all support classes of loaded class use same class loader
        // must check subclass cache of classes for things like Object
        Class result;
        byte  classData[];
        
        // Class loaded yet?
        result = (Class)types.get( name );
        if ( result != null )
            return result;

        
        try
        {
            // try using the primordial class loader...
            if( name.startsWith( "java." ))
                result = super.findSystemClass( name );
            else
                result = super.findClass( name );
            return   result;
        }
        catch( ClassNotFoundException e ){}
        
        // don't load system files ...
        if( name.startsWith( "java." ))
            throw new   ClassNotFoundException();

        // load class from base path
        // Convert class name argument to filename
        // Convert package names into subdirectories
        String filename = name.replace ('.', File.separatorChar) + ".class";
        classData = loadClassData( filename );

        if( classData == null )
        {
            System.out.println( "Simple ClassLoader - Can't load class: " + filename );
            throw new   ClassNotFoundException();
        }

        // parse it...
        result = defineClass( name, classData, 0, classData.length );
        if( result == null )
        {
            System.out.println( "Simple ClassLoader - Format error: " + filename );
            throw new   ClassFormatError();
        }

        if( resolve )
            resolveClass( result );

        // now add type to our cache
        types.put( name, result );

        return result;
   }

   private byte[] loadClassData (String filename)
   {
       // Create a file object relative to directory provided
       File f = new File (root, filename);

       // Get size of class file
       int size = (int)f.length();

       // Reserve space to read
       byte buff[] = new byte[size];

       FileInputStream fis = null;
       DataInputStream dis = null;
       try
       {
           // Get stream to read from
           fis = new FileInputStream(f);
           dis = new DataInputStream (fis);

           // Read in data
           dis.readFully (buff);

           // close stream
           dis.close();
       }
       catch( FileNotFoundException e )
       {
           return null;
       }
       catch( IOException e )
       {
           return null;
       }

       // return data
       return buff;
   }
   
   protected Class findClass( String pClassName ) throws ClassNotFoundException 
   {
       try 
       {
           File classFile = new File( root, pClassName + ".class" );
           InputStream lInput = new BufferedInputStream( new FileInputStream( classFile ) );
           ByteArrayOutputStream lOutput = new ByteArrayOutputStream();
           
           int i = 0;
           while( ( i = lInput.read() ) >= 0 ) 
           {
               lOutput.write( i );
           }
       
           byte[] lBytes = lOutput.toByteArray();
           return defineClass( pClassName, lBytes, 0, lBytes.length );
           
       } 
       catch( Exception e )
       {
           throw new ClassNotFoundException( "Class: " + pClassName + " not found" );
       }
   }
   
   public Class __loadClass( String pClassName, boolean pResolve ) throws ClassNotFoundException 
   {
       Class lClass = findLoadedClass( pClassName );
       if( lClass == null ) 
       {
           try 
           {
               lClass = findClass( pClassName );
           } 
           catch( ClassNotFoundException cnfe ) 
           {
               ClassLoader lParent = getParent();
               if( lParent == null) 
               {
                   lClass = getSystemClassLoader().loadClass( pClassName );
               } 
               else 
               {
                   lClass = lParent.loadClass(pClassName );
               }
           }
       }
       if( pResolve ) 
       {
           resolveClass( lClass );
       }
       return lClass;
   }
}