package s2s.javacourse;


/**
 * Title:        Java Courseware
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      TROy Software Development
 *
 * @author Selvyn Wright
 * @version 1.0
 */
import java.util.Vector;

public class WaitAndNotify_P2
{

    private DispatcherThread dispatcher = new DispatcherThread();
    private WorkerThread worker = new WorkerThread();
    private SharedDataStore dataStore = new SharedDataStore();

    // Use this class to store data in.  The input/outputs are protected from
    // synchronisation errors...
    private class SharedDataStore
    {

        private Vector db = new Vector();
        private boolean jobCompleted = false;

        public synchronized void clearDataStore()
        {
            db.removeAllElements();

            notify();
        }

        public synchronized void updateDB()
        {
            System.out.println("updating the data store");

            for (int i = 0; i < 100; i++)
            {
                db.addElement("Handler One" + i);
            }
            jobCompleted = true;

            System.out.println("update completed");
            notify();
            try
            {
                wait();
            } 
            catch (InterruptedException e)
            {
            }
        }

        // This is a bad design, when this operation exits, the waiting thread now has access
        // to the datastore and the operation updateDB can now be called
        //
        // The call db.clear() above has to be done after the callee of getDB() has finished with
        // the data from this call
        public synchronized Vector getData()
        {
            while (!jobCompleted)
            {
                try
                {
                    wait();
                } 
                catch (InterruptedException e)
                {
                }
            }
            jobCompleted = false;

            return db;
        }
    }

    private class WorkerThread extends Thread
    {
        private boolean jobCompleted = false;
        private Vector db = new Vector();

        public void run()
        {
            for (;;)
            {
                updateDataStore();
            }
        }

        private void updateDataStore()
        {
            dataStore.updateDB();
        }

    }

    private class DispatcherThread extends Thread
    {

        public void run()
        {
            for (int idx = 0; idx < 2; idx++)
            {
                getResults();

                // see note above on design of getData()
                dataStore.clearDataStore();
            }
            System.exit(0);
        }

        private void getResults()
        {
            Vector localDB = dataStore.getData();

            int length = localDB.size();

            for (int i = 0; i < length; i++)
            {
                String val = (String) localDB.elementAt(i);
                System.out.println(val);
            }
        }
    }

    private void execute1()
    {
        System.out.println("execute 1");

        worker.start();
        dispatcher.start();
    }

    private void execute2()
    {
        System.out.println("execute 2");

        dispatcher.start();
        worker.start();
    }

    public static void main(String[] args)
    {
        int version = 1;
        WaitAndNotify_P2 waitAndNotify_P21 = new WaitAndNotify_P2();

        if(version == 2)
        {
            waitAndNotify_P21.execute1();
        } 
        else
        {
            waitAndNotify_P21.execute2();
        }
    }
}
